import { Component, Input, OnInit } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonCompModule } from '../../services/common.modules';
import { Router, ActivatedRoute } from '@angular/router';
import Parse from 'parse';
import { textbookServer } from '../../services/textbook';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzModalService } from 'ng-zorro-antd/modal';
import { MatButtonModule } from '@angular/material/button';
import { NzRadioModule } from 'ng-zorro-antd/radio';
import { DatePipe } from '@angular/common';
import { NzEmptyModule } from 'ng-zorro-antd/empty';
@Component({
  selector: 'app-textbook',
  templateUrl: './textbook.component.html',
  styleUrls: ['./textbook.component.scss'],
  imports: [
    CommonCompModule,
    FormsModule,
    ReactiveFormsModule,
    MatButtonModule,
    NzRadioModule,
    DatePipe,
    NzEmptyModule,
  ],
  standalone: true,
})
export class TextbookComponent implements OnInit {
  textbookList: Array<Parse.Object> = [];
  count: number = 0;
  limit: number = 10;
  pageSize: number = 1;

  @Input('maxWidth') maxWidth: any; //最大宽度
  @Input('depart') depart: any; //指定申报单位
  @Input('recommend') recommend: any; //是否推荐
  @Input('uid') uid: string = ''; //个人空间
  @Input('path') path: string = '/nav-admin/manage/textbook/details';
  @Input('discard') discard: Boolean = false; //是否删除
  //相关权限配置
  @Input('filterObj') filterObj: any = {
    isCheck: true,
    noStared: false, //非推荐
    status: [],
    btns: {
      edit: false, //编辑
      isDelete: false, //删除权限
      review: false, //预览
      reject: false, //退回教材
      star: false, //加入推荐
      remove: false, //移除推荐
      submit: false, //提交
      verify: false, //提交推荐权限
    },
  };

  loading: boolean = false;
  checkedAll: boolean = false; //全选
  setOfCheckedId = new Set<string>();
  searchValue: string = '';
  manage: boolean = false;
  eduProcessId: any = ''; //流程id，verify存在时需要
  constructor(
    public tbookSer: textbookServer,
    private route: Router,
    private activeRoute: ActivatedRoute,
    private message: NzMessageService,
    private modal: NzModalService
  ) {}

  ngOnInit() {
    this.activeRoute.paramMap.subscribe(async (params) => {
      this.eduProcessId = params.get('id');
      this.getTextbook();
      if (Object.values(this.filterObj.btns).some((item) => item)) {
        this.manage = true;
      }
    });
  }
  async getTextbook(val?: string, exported?: boolean): Promise<any[] | void> {
    this.loading = true;
    let query = new Parse.Query('EduTextbook');
    if (val) {
      let query1 = Parse.Query.fromJSON('EduTextbook', {
        where: {
          $or: [
            {
              title: { $regex: `.*${val}.*` },
            },
          ],
        },
      });
      let query2 = Parse.Query.fromJSON('EduTextbook', {
        where: {
          $or: [
            {
              ISBN: { $regex: `.*${val}.*` },
            },
          ],
        },
      });
      query = Parse.Query.or(query1, query2);
    }
    if (this.tbookSer.profile.user.department?.objectId) {
      query.equalTo(
        'department',
        this.tbookSer.profile.user.department?.objectId
      );
    }
    this.depart && query.equalTo('department', this.depart);
    this.recommend && query.equalTo('recommend', true);
    this.filterObj?.noStared && query.notEqualTo('recommend', true);
    this.uid && query.equalTo('user', this.uid);
    query.notEqualTo('isDeleted', true);
    if (!this.uid) {
      query.equalTo('render', true);
      // query.containedIn('status', ['200', '400']);
    }
    if (this.filterObj?.status?.length > 0) {
      query.containedIn('status', this.filterObj?.status);
    }
    if (this.discard) {
      query.equalTo('discard', true);
    } else {
      query.notEqualTo('discard', true);
    }
    if (exported) {
      let r = await query.find();
      this.loading = false;
      return r;
    }
    // query.include('department');
    this.count = await query.count();
    query.limit(this.limit);
    query.skip(this.limit * (this.pageSize - 1));
    this.textbookList = await query.find();
    console.log(this.textbookList);
    this.loading = false;
  }
  onSearch(e: string) {
    console.log(e);
    this.getTextbook(e);
  }
  //分页切换
  pageIndexChange(e: any) {
    console.log(e);
    this.pageSize = e;
    this.getTextbook(this.searchValue);
  }
  //全选
  onAllChecked(checked: boolean) {
    console.log(checked);
    this.textbookList.forEach((item: any) => {
      if (checked) {
        this.setOfCheckedId.add(item.id);
      } else {
        this.setOfCheckedId.delete(item.id);
      }
    });
    if (!checked) {
      this.setOfCheckedId = new Set<string>();
    }
    this.checkedAll = checked;
  }
  //单选
  onItemChecked(id: string, e: boolean) {
    if (e) {
      this.setOfCheckedId.add(id);
    } else {
      this.setOfCheckedId.delete(id);
    }
    this.checkedAll = this.textbookList.every((item) =>
      this.setOfCheckedId.has(item.id)
    );
  }
  reject(data?: Parse.Object) {
    this.modal.confirm({
      nzTitle: '批量退回',
      nzContent: `确定退回吗？`,
      nzOkText: '确认',
      nzOkType: 'primary',
      nzOkDanger: true,
      nzOnOk: async () => {
        if (data?.id) {
          data.set('status', '300');
          await data.save();
        } else {
          let selectedList = this.textbookList.filter((item: any) =>
            this.setOfCheckedId.has(item?.id)
          );
          let deletePromiseList = selectedList.map((item: any) => {
            return new Promise((resolve) => {
              item.set('status', '300');
              item.save().then(() => resolve(true));
            });
          });
          try {
            await Promise.all(deletePromiseList);
          } catch (err) {}
        }
        this.getTextbook();
      },
      nzCancelText: '取消',
      nzOnCancel: () => console.log('Cancel'),
    });
  }
  //删除
  del(data?: Parse.Object) {
    console.log(data);
    this.modal.confirm({
      nzTitle: '删除教材',
      nzContent: `删除后数据不可恢复，请谨慎操作`,
      nzOkText: '确认',
      nzOkType: 'primary',
      nzOkDanger: true,
      nzOnOk: async () => {
        if (data?.id) {
          data.set('isdeleted', true);
          await data.save();
        } else {
          let selectedList = this.textbookList.filter((item: any) =>
            this.setOfCheckedId.has(item?.id)
          );
          let deletePromiseList = selectedList.map((item: any) => {
            return new Promise((resolve) => {
              item.set('isdeleted', true);
              item.save().then(() => resolve(true));
            });
          });
          try {
            await Promise.all(deletePromiseList);
          } catch (err) {}
        }
        this.getTextbook();
      },
      nzCancelText: '取消',
      nzOnCancel: () => console.log('Cancel'),
    });
  }
  //加入/移除推荐
  updateStatus(data?: Parse.Object, recommend?: boolean) {
    this.modal.confirm({
      nzTitle: `${recommend ? '加入' : '移除'}推荐`,
      nzContent: `确定${recommend ? '加入' : '移除'}推荐吗？`,
      nzOkText: '确认',
      nzOkType: 'primary',
      nzOkDanger: !recommend,
      nzOnOk: async () => {
        if (data?.id) {
          data.set('recommend', recommend);
          await data.save();
          this.getTextbook();
        } else {
          let selectedList = this.textbookList.filter((item: any) =>
            this.setOfCheckedId.has(item?.id)
          );
          let deletePromiseList = selectedList.map((item: any) => {
            return new Promise((resolve) => {
              item.set('recommend', recommend);
              item.save().then(() => resolve(true));
            });
          });
          try {
            await Promise.all(deletePromiseList);
            this.getTextbook();
            history.go();
          } catch (err) {}
        }
      },
      nzCancelText: '取消',
      nzOnCancel: () => console.log('Cancel'),
    });
  }
  //提交教材
  submit(data?: Parse.Object) {
    console.log(data);
    this.modal.confirm({
      nzTitle: '提交教材',
      nzContent: `提交后在已提交评审教材中查看`,
      nzOkText: '确认',
      nzOkType: 'primary',
      nzOkDanger: false,
      nzOnOk: async () => {
        if (data?.id) {
          let t =
            this.tbookSer.formatTime('YYYYmmdd', new Date()) +
            Math.random().toString().slice(-4);
          data.set('render', true);
          data.set('status', '200');
          data.set('code', t);
          await data.save();
        } else {
          let selectedList = this.textbookList.filter((item: any) =>
            this.setOfCheckedId.has(item?.id)
          );
          let deletePromiseList = selectedList.map((item: any) => {
            return new Promise((resolve) => {
              let t =
                this.tbookSer.formatTime('YYYYmmdd', new Date()) +
                Math.random().toString().slice(-4);
              item.set('render', true);
              item.set('status', '200');
              item.set('code', t);
              item.save().then(() => resolve(true));
            });
          });
          try {
            await Promise.all(deletePromiseList);
          } catch (err) {}
        }
        this.getTextbook();
      },
      nzCancelText: '取消',
      nzOnCancel: () => console.log('Cancel'),
    });
  }
  toUrl(url: string, param?: Object) {
    console.log(url);
    if (param) {
      this.route.navigate([url, param]);
      return;
    }
    this.route.navigate([url]);
  }

  //导出
  async export() {
    let data: any = await this.getTextbook('', true);
    let table = `<table border="1px" cellspacing="0" cellpadding="0">
        <thead>
          <tr>
            <th>申报编号</th>
            <th>教材名称</th>
            <th>册数</th>
            <th>第一主编/作者</th>
            <th>所属单位</th>
            <th>所属本科专业</th>
            <th>主要语种类型</th>
            <th>ISBN</th>
            <th>出版单位</th>
          </tr>
        </thead>
        <tbody>
        `;
    let _body = '';
    for (var row = 0; row < data.length; row++) {
      _body += '<tr>';
      _body += '<td>';
      _body += `${row + 1}`;
      _body += '</td>';

      _body += '<td>';
      _body += ` &nbsp;${data[row].get('title') || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${
        data[row]?.get('type') == '单册'
          ? '单册'
          : data[row]?.get('typeNumber') || '-'
      }`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('author') || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('unit') || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('major').name || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('lang') || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('ISBN') || '-'}`;
      _body += '</td>';

      _body += '<td>';
      _body += `${data[row]?.get('editionUnit') || '-'}`;
      _body += '</td>';
      _body += '</tr>';
    }
    table += _body;
    table += '</tbody>';
    table += '</table>';
    let title = '申报流程列表';
    this.excel(table, `${title}.xls`);
  }
  excel(data: any, filename: string) {
    let html =
      "<html xmlns:o='urn:schemas-microsoft-com:office:office' xmlns:x='urn:schemas-microsoft-com:office:excel' xmlns='http://www.w3.org/TR/REC-html40'>";
    html +=
      '<meta http-equiv="content-type" content="application/vnd.ms-excel; charset=UTF-8">';
    html += '<meta http-equiv="content-type" content="application/vnd.ms-excel';
    html += '; charset=UTF-8">';
    html += '<head>';
    html += '</head>';
    html += '<body>';
    html += data;
    html += '</body>';
    html += '</html>';
    let uri =
      'data:application/vnd.ms-excel;charset=utf-8,' + encodeURIComponent(html);
    let link = document.createElement('a');
    link.href = uri;
    link.download = `${filename}`;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
